/**
 *******************************************************************************
 * @file    lcd061.c
 * @brief   The functions to use LCD module in TMPM061-SK board.
 * @version V0.102
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
 
#include "lcd061.h"
#include "tmpm061_cg.h"
#include <stdio.h>


const uint8_t number_font_table[] = {
    0x3F,   /* digit '0' */  
    0x06,   /* digit '1' */
    0x5B,   /* digit '2' */
    0x4F,   /* digit '3' */
    0x66,   /* digit '4' */
    0x6D,   /* digit '5' */
    0x7D,   /* digit '6' */   
    0x07,   /* digit '7' */  
    0x7F,   /* digit '8' */
    0x6F    /* digit '9' */
};                                   


/**
  * @brief  Clear all LCD buffer. 
  * @param  None
  * @retval None
  */
void lcd_clear(void)
{
    LCD_BufIndex tmp = LCD_BUF_SEG0100;
    
    for(; tmp < LCD_BUF_SEG3938; tmp++) {
        LCD_WriteBuf(tmp, 0);
    }
}

/**
  * @brief  Initialize GPIO as LCD driver pins. 
  * @param  None
  * @retval None
  */
void lcd_port_init(void)
{
    /* Port init to SEG and COM */
    TSB_PA->FR1 = 0xFFU;
    TSB_PB->FR1 = 0xFFU;
    TSB_PC->FR1 = 0xFFU;
    TSB_PD->FR1 = 0xFFU;
}

/**
  * @brief  Configure LCD modules. 
  * @param  None
  * @retval None
  */
void lcd_configure_init(void)
{

    /* enable LCD module */
    LCD_Enable();   
    
    /* set LCD duty and bias */
    LCD_SetDutyBias(LCD_STATIC);           /* static LCD on M061-SK board */
    
    /* set frame frequency */
    LCD_SetBaseFreq(LCD_FS_DIVIDE_2_POWER_8);   /* F_base = fs / (2^8) */
    
    /* Set bleeder resistor source as internal */
    LCD_SetBleederSource(LCD_BLEEDER_RESISTOR_INTERNAL);
    
    /* Set high internal bleeder resistor value */
    LCD_SetInternalBleeder(LCD_BLEEDER_RESISTOR_500K);

    /* Set low internal bleeder resistor connection time */
    LCD_SetLowBleederTime(LCD_2_POWER_6_DEVIDE_BASE_FREQ);
    
    /* clear LCD buffer */   
    lcd_clear();
    
    /* LCD on */
    LCD_SetDisplay(ENABLE);
  
}
/*******************************************************************************
* Function Name  : LCD_Configuration
* Description    : Configures LCD
* Input          : None.
* Return         : None.
*******************************************************************************/
void LCD_Init(void)
{
    /* enable low-speed oscillator for LCD */
    if(CG_GetFsState() == DISABLE)
    {
        CG_SetFs(ENABLE);
    }
    lcd_port_init();
    lcd_configure_init();
}

/**
  * @brief  Display 4 digit on LCD panel which can display"8.8.8.8"
  * @param  p: pointer point to the string which will be displayed. 
  * @param  dot_pos: The position of dot, count from left, range: 1 to 3, 0 is not display dot.
  * @retval None
  */
void display_4digit(unsigned char *p, uint8_t dot_pos)
{
    uint8_t tmp = 0U;
    uint8_t loop = 0U;
    uint8_t idx = 0U, dat = 0U;  
    
    LCD_BufIndex buf_head = LCD_BUF_SEG0100;
    
    for( ; loop < 4U; loop++) { /* total 4 digit need to display */
        tmp = *(p + loop);
        if( tmp == ' ' ) {      /* is SPACE, hide it */
            tmp = 0U;
        } else {
            tmp = number_font_table [ tmp - '0' ];
            if( dot_pos == loop + 1U) { /* if dot_pos isn't '0', should display dot */
                tmp |= 0x80U;
            } else {
                /* Do nothing */
            }
        }

        /* now 'tmp' store the font data, we will set its 8 bit info to 8 LCD Segment buffer */
        for(idx = 0U; idx < 4U; idx++) {
            dat = 0U;
            if( tmp & 0x01U ) {
                dat |= 0x01U;
            } else { 
                /* Do nothing */
            }
            if( tmp & 0x02U ) {
                dat |= 0x10U;
            } else { 
                /* Do nothing */
            }
            LCD_WriteBuf( (LCD_BufIndex)(buf_head + idx) , dat );   /* set 2 segment at one time */
            tmp >>= 2U;
         }
         buf_head += 4U;    /* for the static LCD, one digit '8' need 8 seg, */
                            /*  but we process 2 seg at one time, so, only need to add 4 here */
    }

}

